"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
var _a = require('winston'), createLogger = _a.createLogger, format = _a.format, transports = _a.transports;
var chalk = require('chalk');
var combine = format.combine, printf = format.printf;
var myFormatter = format(function (info) {
    info.timestamp = new Date(Date.now()).toISOString();
    //Check logging level
    var levelUpper = info.level.toUpperCase();
    switch (levelUpper) {
        case 'INFO':
            info.message = chalk.green(info.message);
            info.level = chalk.black.bgGreenBright.bold(info.level);
            break;
        case 'WARN':
            info.message = chalk.yellow(info.message);
            info.level = chalk.black.bgYellowBright.bold(info.level);
            break;
        case 'ERROR':
            info.message = chalk.red(info.message);
            info.level = chalk.black.bgRedBright.bold(info.level);
            break;
        default:
            break;
    }
    return info;
})();
var Logger = /** @class */ (function () {
    function Logger(root, functionName) {
        var local_timestamp = new Date(Date.now());
        var date = local_timestamp.toISOString().split('T')[0];
        var time = local_timestamp.toISOString().split('T')[1].split('.')[0];
        var dateTime = (date + '_' + time).replaceAll(":", ".").replaceAll("-", ".");
        ;
        var name = "".concat(functionName).toString().split('.')[0];
        var logPath = "".concat(root, "/logs/").concat(name, "/").concat(dateTime, ".log");
        console.log("Log Path: " + logPath);
        this._logger = createLogger({
            level: 'info',
            format: combine(myFormatter, printf(function (info) {
                return "[".concat(chalk.rgb(125, 125, 125)(info.timestamp), "] [").concat(info.level, "]: ").concat(info.message);
            })),
            transports: [
                new transports.Console(),
                new transports.File({
                    filename: logPath,
                    maxsize: '5000000',
                    maxFiles: '1000',
                    format: combine(myFormatter, printf(function (info) {
                        return "[".concat(chalk.black.bgWhiteBright(info.timestamp), "] [").concat(info.level, "]: ").concat(info.message);
                    })),
                })
            ],
        });
    }
    Logger.prototype.log = function (msg) {
        this._logger.info(msg);
    };
    Logger.prototype.error = function (msg) {
        this._logger.error(msg);
    };
    Logger.prototype.warn = function (msg) {
        this._logger.warn(msg);
    };
    return Logger;
}());
exports.Logger = Logger;
